import $ from 'jquery'
import { sticky } from './adaptiveSticky'
import { Plugin } from 'foundation-sites/js/foundation.core.plugin'
import { MediaQuery } from 'foundation-sites/js/foundation.util.mediaQuery'
import { debounce } from '@moreonion/js-utils/src/utils'

/**
 * AdaptiveSticky module.
 *
 * @module foundation.adaptiveSticky
 */

class AdaptiveSticky extends Plugin {
  /**
   * Creates a new instance.
   * @class
   * @name AdaptiveSticky
   * @fires AdaptiveSticky#init
   * @param {Object} element - jQuery object to add the trigger to.
   * @param {Object} options - Overrides to the default plugin settings.
   */
  _setup(element, options = {}) {
    this.$element = element
    this.className = 'AdaptiveSticky'
    // defaults, fallbacks
    this.$target = element
    this.mediaQuery = '' // always true for MediaQuery.is()
    // true when adaptive to viewport height, i.e. maxHeight == window.innerHeight
    this.autoMaxHeight = true
    this.maxHeight = window.innerHeight

    this.options = $.extend(true, {}, AdaptiveSticky.defaults, this.$element.data(), options)

    // set target value, if given a correct selector
    if (this.options.target) {
      const $target = $(this.options.target, this.$element)
      if ($target.length) {
        this.$target = $target
      }
    }

    // set mediaQuery value, if given a correct type
    if (this.options.mediaQuery && typeof this.options.mediaQuery === 'string') {
      this.mediaQuery = this.options.mediaQuery
    }

    // set maxHeight value, if given a correct type
    if (this.options.maxHeight && typeof this.options.maxHeight === 'number') {
      this.maxHeight = this.options.maxHeight
      this.autoMaxHeight = false
    }

    // Ensure initialized MediaQuery so that MediaQuery.current is set.
    // We need this when calling sticky().
    // https://github.com/foundation/foundation-sites/issues/10363
    // Probably needed because of race condition in `.foundation()` calling
    // code?
    MediaQuery._init()

    this._init()
  }

  /**
   * Initializes the plugin and calls functions to get functioning on load.
   * @private
   */
  _init() {
    // Debounced function to update the sticky.
    // The main change we want to capture with this is the maxHeight of the
    // sticky.
    this._debouncedUpdate = debounce(() => {
      if (this.autoMaxHeight) {
        this.maxHeight = window.innerHeight
      }

      sticky(this.$element[0], 'update', {
        offset: this.options.offset,
        target: this.$target[0],
        maxHeight: this.maxHeight,
        debug: this.options.debug
      })
    }, 300)

    // Debounced function to set the stickyness of the sticky.
    // Depends on the mediaQuery setting.
    this._debouncedStick = debounce(() => {
      if (MediaQuery.is(this.mediaQuery)) {
        sticky(this.$element[0], 'stick', {
          offset: this.options.offset,
          target: this.$target[0],
          maxHeight: this.maxHeight,
          debug: this.options.debug
        })
      } else {
        sticky(this.$element[0], 'unstick')
      }
    }, 100)

    // Initial call to determine of we are sticky or not.
    // Checks the MediaQuery and calls sticky() accordingly.
    this._debouncedStick()

    this._listeners()
  }

  /**
   * Outputs sticky status.
   * @private
   */
  _printStatus() {
    console.debug(sticky(this.$element[0], 'status'))
  }

  /**
   * Updates sticky.
   * @private
   */
  _update(options) {
    sticky(this.$element[0], 'update', options)
  }

  /**
   * Initializes listeners.
   * @private
   */
  _listeners() {
    // Update on window resizes
    window.addEventListener('resize', this._debouncedStick)
    window.addEventListener('resize', this._debouncedUpdate)
  }

  /**
   * Destroys an instance.
   */
  _destroy() {
    window.removeEventListener('resize', this._debouncedStick)
    window.removeEventListener('resize', this._debouncedUpdate)
    sticky(this.$element[0], 'unstick')
  }
}

/**
 * Default settings for plugin.
 */
AdaptiveSticky.defaults = {
  /**
   * The target at which to stick to.
   * @option
   * @type {Element|null}
   * @default null
   */
  target: null,

  /**
   * A value to visually offset the stickyness.
   *
   * Calculated based on the targets position.
   * @option
   * @type {number}
   * @default 0
   */
  offset: 0,

  /**
   * A px value how big the sticky is allowed to be.
   *
   * If it is taller, it will be unsticked.
   * @option
   * @type {number|null}
   * @default null
   */
  maxHeight: null,

  /**
   * The breakpoint at which we un/stick.
   *
   * String values correspond to Foundation media queries like
   * "medium up", ...
   * @option
   * @see https://get.foundation/sites/docs/media-queries.html#javascript
   * @type {string|null}
   * @default null
   */
  mediaQuery: null,

  /**
   * Toggle debug output.
   * @option
   * @type {boolean}
   * @default false
   */
  debug: false
}

export { AdaptiveSticky, sticky }
